/******************************************************************************
*******************************************************************************
**
**   MODULE      VISADev.CPP
**
**   PROJECT     several
**
**   COMPILER    VisualStudio2003
**
**   DATE        2008-01-10
**
**   AUTHOR      T. Braunstorfinger 1GP1
**
**   ABSTRACT    Driving a VISA instrument - class interface for VISA
**
**   HISTORY     2008-01-10 first version, based on GpibDev (1995/1998)
**
*******************************************************************************
******************************************************************************/

/* INCLUDE FILES *************************************************************/

/* IMPORT */
#include "stdafx.h"
#include "math.h"

/* EXPORT */
#include ".\visadev.h"

/* GLOBAL VARIABLES **********************************************************/

/* GLOBAL DEFINES ************************************************************/

/* LOCAL DEFINES *************************************************************/

#define BUFFER_SIZE 10000

/* LOCAL VARIABLES ***********************************************************/

//initialize static members
CStdioFile CVISADev::m_Debugfile;
bool CVISADev::m_bDebugfileIsOpen = false;
int CVISADev::m_nInstanceCount = 0;

/* LOCAL CONSTANTS ***********************************************************/
    
/* CLASS CVISADev ************************************************************/    

/* FUNCTION ******************************************************************/

CVISADev::CVISADev(CString sVisaResourceString, bool bFileDump)
                      
/*
SPECIFICATION :
Construktor
PARAMETERS :
CString sVisaResourceString : 
for example : GPIB0::28::INSTR, TCPIP::10.111.10.221::INSTR, USB:: ...
It's implemented in a tolerant way, so also abbreviations are possible
Number below 32 -> GPIB-Address
names, ip-adresses w/o TCPIP:: ::INSTR -> extended
bool bFileDump : if true, all transactions are logged to VISADev.log

SIDE_EFFECTS :
RETURN VALUE :
******************************************************************************/
{
	TRACE("CVISADev::CVISADev");
    
	char buffer[BUFFER_SIZE];
	ViUInt32 retCount;

   	m_nInstanceCount ++;

	m_bDeviceReady = FALSE;
    m_lTotalMessageCount = 0L;
	
	/* Begin by initializing the system*/
	status = viOpenDefaultRM(&defaultRM);
	if (status < VI_SUCCESS) 
	{
		/* Error Initializing VISA...exiting*/
		AfxMessageBox("VISA Init Error",  MB_ICONEXCLAMATION);
		return;
	}

	// store resource estring
	m_sVisaResourceString = sVisaResourceString;

	//Expand String to correct format
	if (m_sVisaResourceString.Find("::INSTR") == -1)
	{
		//invalid format, try to repair
		if (m_sVisaResourceString.GetLength() == 2)
		{
			//probably a naked GPIB-address
			m_sVisaResourceString = "GPIB0::"+ m_sVisaResourceString + "::INSTR";
		}
		else
		{
			//try to interpret as network address
			m_sVisaResourceString = "TCPIP::"+ m_sVisaResourceString + "::INSTR";
		}
	}

	/* Open communication with Device */
	status = viOpen(defaultRM, (ViRsrc)(const char *)m_sVisaResourceString, VI_NULL, VI_NULL,&instr);
	if (status < VI_SUCCESS) 
	{
		/* Error finding device */
		AfxMessageBox("Cannot open device "+m_sVisaResourceString,  MB_ICONEXCLAMATION);
		return;
	}
	
	/* ok, we're connected ... to get sure, read *IDN */
	/* Ask the device for identification */
	status = viWrite(instr, (ViBuf)"*IDN?\n", 6, &retCount);
	status = viRead(instr, (ViPBuf)buffer, BUFFER_SIZE, &retCount);
	if (status < VI_SUCCESS) 
	{
		/* Error reading drom  device */
		AfxMessageBox("Cannot read *IDN from device "+m_sVisaResourceString,  MB_ICONEXCLAMATION);
		return;
	}
	
	//when even *IDN reading was successful, we should be really ready :-)
	m_bDeviceReady = true;

	/* Set the timeout for message-based communication*/
	status = viSetAttribute(instr, VI_ATTR_TMO_VALUE, 10000);

	if (bFileDump)
	{
		//open dumpfile (if we are the first)
		if (!m_bDebugfileIsOpen)
			m_Debugfile.Open("VISADev.log", CFile::modeWrite | CFile::modeCreate);
		
		m_bDebugfileIsOpen = true;
	}
}


/* FUNCTION ******************************************************************/
                      
CVISADev::~CVISADev()
                      
/*
SPECIFICATION :
Destructor
PARAMETERS :
SIDE_EFFECTS :
RETURN VALUE :
******************************************************************************/
{                             
    //close Debug file, if this is the last destructor call
	m_nInstanceCount --;
	if (!m_nInstanceCount && m_bDebugfileIsOpen)
	{	
		m_Debugfile.Close();
		m_bDebugfileIsOpen = false;
	}

	TRACE("CVISADev::~CVISADev (%ld)", m_lTotalMessageCount);

	/* Close down the system */
	status = viClose(instr);
	status = viClose(defaultRM);
}

bool CVISADev::IsDeviceReady(void)
{
	return m_bDeviceReady;
}

/* FUNCTION ******************************************************************/
                      
bool CVISADev::Write(CString sCommand, bool bShowError)
                      
/*
SPECIFICATION :
Write a string to the device (incl. error handling)

PARAMETERS :                                   
sName : String to be written
bShowError : Supressing error reporting is possible
SIDE_EFFECTS :
RETURN VALUE :
FALSE : OK!
TRUE  : Error
******************************************************************************/
{
	char buffer[BUFFER_SIZE];
	ViUInt32 retCount;

    if (m_bDeviceReady)
    {
        TRACE("To VISA device %s -> %s\r\n", m_sVisaResourceString, sCommand);

		//Append line feed
		sCommand += '\n';
		status = viWrite(instr, (ViBuf)(const char*) sCommand, sCommand.GetLength(), &retCount);
		
		//make a nice entry to the log file containig also a time stamp
		if (m_bDebugfileIsOpen)
	    {
	        CTime t = CTime::GetCurrentTime();
			sprintf(buffer, "%02d/%02d/%02d-%02d:%02d:%02d : (status=%d) to %s >%s<\n",(t.GetYear())%100, t.GetMonth(), t.GetDay(),
								    t.GetHour(), t.GetMinute(), t.GetSecond(), status, m_sVisaResourceString, sCommand);

		    m_Debugfile.WriteString(buffer);
		}

		//optionally read the error queue to get a fast feed back
		if (bShowError)
			ReadErrorQueue(true);

		if (status < VI_SUCCESS) 
		{
			TRACE("Error : status = %d\r\n", status);
			
			//important to avoid endless errors...
			m_bDeviceReady = FALSE;

			return true;
		}
	}
        
	return false;
}


/* FUNCTION ******************************************************************/
                      
CString CVISADev::Read(bool bShowError)
                      
/*
SPECIFICATION :
Reads a message from a VISA device
PARAMETERS :                                   
bShowError : Supressing error reporting is possible
SIDE_EFFECTS :
RETURN VALUE :
read message
******************************************************************************/
{       
    char buffer[BUFFER_SIZE];
	char dummy[BUFFER_SIZE];
	ViUInt32 retCount;

    static CString s;  // its static because it would be freed after leaving scope/method
    
    if (m_bDeviceReady)
    {
		status = viRead(instr, (ViPBuf)buffer, BUFFER_SIZE, &retCount);
		if (status < VI_SUCCESS) 
		{
			/* Error reading from device */
			TRACE("Error Reading from VISA device %s !\r\n", m_sVisaResourceString);
		    dummy[0] = 0;

			AfxMessageBox("Cannot read from device "+m_sVisaResourceString,  MB_ICONEXCLAMATION);
			
			//important to avoid endless errors...
			m_bDeviceReady = FALSE;
			return CString("");
		}   
	    else
	    {
	        //OK!
	        buffer[retCount-1] = 0;
	        TRACE("From VISA device %s : %s\r\n", m_sVisaResourceString, buffer);	        
	    }
	    
	    s = CString(buffer);
	    
	    //make a nice entry to the log file containig also a time stamp
		if (m_bDebugfileIsOpen)
	    {
	        CTime t = CTime::GetCurrentTime();
			sprintf(dummy, "%02d/%02d/%02d-%02d:%02d:%02d : (status=%d) from %s >%s<\n",  (t.GetYear())%100, t.GetMonth(), t.GetDay(),
								    t.GetHour(), t.GetMinute(), t.GetSecond(), status, m_sVisaResourceString, buffer);

		    m_Debugfile.WriteString(dummy);
		}    	                        
		
		//optionally read the error queue to get a fast feed back
		if (bShowError)
			ReadErrorQueue(true);

	    m_lTotalMessageCount++;
	}
	    
    return s;
}

/* FUNCTION ******************************************************************/
                      
CString CVISADev::Request(CString sCommand, bool bShowError)
                      
/*
SPECIFICATION :
Query and read answer within one single command
PARAMETERS :                                                        
sCommand   : Befehlstring
bShowError : Unterdrueckung der Fehlermeldung moeglich
SIDE_EFFECTS :
RETURN VALUE :
eingelesener Text
******************************************************************************/
{
    if (!Write (sCommand, bShowError))
        return (Read(bShowError));
    else
        return ("");
}

/* FUNCTION ******************************************************************/

bool CVISADev::ReadErrorQueue(bool bShowContents)

/*
SPECIFICATION :
Flushes Error Queue(s)
PARAMETERS :                                                        
bool bShowContents : true -> MessageBox, false -> silent
SIDE_EFFECTS :
RETURN VALUE :
true -> at least one error
false -> no error
******************************************************************************/
{
	CString sErrorText = "";
	CString sErrorTextComplete = "";

	do 
	{
		sErrorText = Request("SYST:ERR?", false);
		if (sErrorText.GetAt(0) != '0')
			sErrorTextComplete += "\n" + sErrorText;

		//Spezialbhandlung ADVANTEST
		if (sErrorText.Left(10) == "-999999999")
			sErrorText.SetAt(0, '0');

	}
	while (sErrorText.GetAt(0) != '0');
	
	if (!sErrorTextComplete.IsEmpty() && bShowContents)
		AfxMessageBox("Error Queue of "+m_sVisaResourceString+":\n" + sErrorTextComplete,  MB_ICONEXCLAMATION);
	
	return true;
}


/* FUNCTION ******************************************************************/
                      
bool CVISADev::CopyFile2Device(CString sFilename, CString sTargetDir, bool bShowError)
                      
/*
SPECIFICATION :
Writes a file to the device (incl. error handling)

PARAMETERS :                                   
sFilename : Name of the file
sTargetDir : Directory to place the file on target
bShowError : Supressing error reporting is possible
SIDE_EFFECTS :
RETURN VALUE :
FALSE : OK!
TRUE  : Error
******************************************************************************/
{
	char buffer[BUFFER_SIZE];
	ViUInt32 retCount;
	CString sCommand;


    if (m_bDeviceReady)
    {
        TRACE("Copy file %s To VISA device %s at target directory %s \r\n", sFilename, m_sVisaResourceString, sTargetDir);

		//Switch target directory
		sCommand = "MMEM:CDIR '" + sTargetDir + "'";
		Write(sCommand);
		
		//Open File and read length
		CFile TransferFile(sFilename, CFile::modeRead);
		int nFileLength = (int)TransferFile.GetLength();

		//does file fit into buffer?
		if (nFileLength > BUFFER_SIZE - 100)
		{
			TRACE("Error : file too big for a small little buffer\r\n");
			return true;
		}

		sprintf(buffer,"MMEM:DATA \'%s\',#%d%d", (const char*)sFilename,1+(int)log10((double)nFileLength), nFileLength);

		int nHeaderLength = (int)strlen(buffer);
		TransferFile.Read(buffer+nHeaderLength, nFileLength);
		TransferFile.Close();

		status = viWrite(instr, (ViBuf)(buffer), nHeaderLength+nFileLength, &retCount);
		
		//make a nice entry to the log file containig also a time stamp
		if (m_bDebugfileIsOpen)
	    {
	        CTime t = CTime::GetCurrentTime();
			sprintf(buffer, "%02d/%02d/%02d-%02d:%02d:%02d : (status=%d) to %s CopyFile>%s<\n",(t.GetYear())%100, t.GetMonth(), t.GetDay(),
								    t.GetHour(), t.GetMinute(), t.GetSecond(), status, m_sVisaResourceString, sFilename);

		    m_Debugfile.WriteString(buffer);
		}

		//optionally read the error queue to get a fast feed back
		if (bShowError)
			ReadErrorQueue(true);

		if (status < VI_SUCCESS) 
		{
			TRACE("Error : status = %d\r\n", status);
			
			//important to avoid endless errors...
			m_bDeviceReady = FALSE;

			return true;
		}
	}
        
	return false;
}
