#include <QMessageBox>
#include <QTime>
#include <math.h>

#include "SCPIsocket.h"

#define BLOCK_SIZE 10000

SCPIsocket::SCPIsocket(const QString & hostName)
{
    //create socket
    tcpSocket = new QTcpSocket(this);
    m_isConnected = false;

    //Connect socket events(=SIGNALSs) to callbacks(=SLOTs)
    //QObject::connect(tcpSocket, SIGNAL(readyRead()), this, SLOT(readData()));
    QObject::connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),
            this, SLOT(displayError(QAbstractSocket::SocketError)));
    QObject::connect(tcpSocket, SIGNAL(connected()), this, SLOT(connected()));

    //connect to host
    connectToHost(hostName);

}

SCPIsocket::SCPIsocket(void)
{
    //create socket
    tcpSocket = new QTcpSocket(this);
    m_isConnected = false;

    //Connect socket events(=SIGNALSs) to callbacks(=SLOTs)
    //QObject::connect(tcpSocket, SIGNAL(readyRead()), this, SLOT(readData()));
    QObject::connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),
            this, SLOT(displayError(QAbstractSocket::SocketError)));
    QObject::connect(tcpSocket, SIGNAL(connected()), this, SLOT(connected()));

}

SCPIsocket::~SCPIsocket()
{
    delete tcpSocket;
    tcpSocket = NULL;
}

void SCPIsocket::connectToHost(const QString & hostName)
{
    //connect to host
    tcpSocket->abort();
    tcpSocket->connectToHost(hostName,5025);
}

//Slot for successful connection
void SCPIsocket::connected(void)
{
    //Request ID for status display
    tcpSocket->write("*IDN?\n");

    //The reply will be very fast, so we can afford to read synchonously
    if (tcpSocket->waitForReadyRead(100))
    {
        char data[1000];
        tcpSocket->readLine(data, 999);
        m_isConnected = true;
    }    
}

//Slot for available socket data (asynchronous reads)
//Not used here, because we are using synchronous reads
void SCPIsocket::readData(void)
{
}

//Slot for error handling of socket
void SCPIsocket::displayError(QAbstractSocket::SocketError socketError)
{
    switch (socketError)
    {
        case QAbstractSocket::RemoteHostClosedError:
            break;
        case QAbstractSocket::HostNotFoundError:
            QMessageBox::information(NULL, "SCPI Socket",
                                        "The device was not found. Please check the "
                                        "device address and try again.");
            break;
        case QAbstractSocket::ConnectionRefusedError:
            QMessageBox::information(NULL, "SCPI Socket",
                                        "The connection was refused by the peer. "
                                        "Make sure the device is running, "
                                        "and check that the device address"
                                        "is correct.");
            break;
        default:
            QMessageBox::information(NULL, "SCPI Socket",
                                     QString("The following error occurred: %1.")
                                     .arg(tcpSocket->errorString()));
    }

}

//Public functions
qint64 SCPIsocket::write ( const QString & data, bool suppressLF )
{
    qDebug(data.toLatin1());
    //Only a conversion to QByteArray
    qint64 bytes = tcpSocket->write(data.toAscii());

    //append a LF
    if (!suppressLF)
        tcpSocket->write("\n");

    return bytes+1;
}

qint64 SCPIsocket::write ( const char * data, qint64 maxSize, bool suppressLF )
{
    //fully transparent
    qint64 bytes = tcpSocket->write(data, maxSize);

    //append a LF
    if (!suppressLF)
        tcpSocket->write("\n");

    return bytes+1;
}

qint64 SCPIsocket::read ( char * data, qint64 maxSize, int waitBeforeReadMs )
{
    if (waitBeforeReadMs)
        tcpSocket->waitForReadyRead(1000);
    qint64 readBytes = tcpSocket->read(data, maxSize);
    data[readBytes] = 0;

    qDebug(data);
    return readBytes;
}

QString SCPIsocket::read(qint64 maxSize, int waitBeforeReadMs)
{
    if (waitBeforeReadMs)
        tcpSocket->waitForReadyRead(1000);
    QByteArray buffer = tcpSocket->read(maxSize);

    qDebug(((QString)buffer).toLatin1());
    return (QString) buffer;
}

QString SCPIsocket::request(const QString & data, qint64 maxSize, int waitBeforeReadMs)
{
    write (data);
    return read(maxSize, waitBeforeReadMs);
}

bool SCPIsocket::readErrorQueue(bool showErrors)
{
    bool isError = false;

    QString allErrors = "";
    QString error;
    bool lastEntry;

    do
    {
        error = request("SYST:ERR?", 1000);

        QString error2 = error.toLower();
        lastEntry = error2.contains("0,\"no error\"");
        if (!lastEntry)
        {
            allErrors += error + "\n";
            isError = true;
        }
    }
    while (!lastEntry);

    if (isError && showErrors)
       QMessageBox::information(NULL, "Error from device", allErrors);

    return isError;
}

qint64 SCPIsocket::waitForData (int waitBeforeReadMs)
{
    if (waitBeforeReadMs)
        tcpSocket->waitForReadyRead(1000);
    return tcpSocket->bytesAvailable();
}
