#include <QFileDialog>
#include <QMessageBox>
#include <QTime>
#include <QTimer>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "TargetDirBrowser.h"
#include <math.h>
#include "SCPIsocket.h"

#define BLOCK_SIZE 10000

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    tcpSocket = new SCPIsocket();
    isLINUX = false;
    fileName = "";
    targetDir = "";
}

MainWindow::~MainWindow()
{
    delete ui;
    delete tcpSocket;
}

//If the connect button is pressed, a connection is established at port 5025
void MainWindow::connectButtonPressed(void)
{
    blockSize = 0;
    QString hostName = ui->deviceAddress->text();
    tcpSocket->connectToHost(hostName);
    ui->status->setPlainText("no connection");
    isConnected = false;
    QTimer::singleShot(1000, this, SLOT(connectionCheck()));
}

void MainWindow::connectionCheck(void)
{
    if (tcpSocket->isConnected())
    {
        ui->status->setPlainText("connected");

        //Request ID for status display
        QString IDNstring = tcpSocket->request("*IDN?", 1000);

        ui->status->setPlainText(IDNstring);
        isConnected = true;

        //Check, whether connect instrument uses LINUX OS
        isLINUX = false;
        if (IDNstring.contains("SMA100A") || IDNstring.contains("SMB100A") ||
            IDNstring.contains("SMC100A") || IDNstring.contains("SMF100A") ||
            IDNstring.contains("SMBV100A")|| IDNstring.contains("NRP-ZLAN"))
            isLINUX = true;
    }
    else
        ui->status->setPlainText("connection error");
}

//If the browse button is pressed, the user has to select the file to transmit
void MainWindow::browseButtonPressed(void)
{
    fileName = QFileDialog::getOpenFileName(this, "Select File to Transmit",
                                                 "",
                                                 "All files (*.*)");
    //display the selection
    ui->sourceFile->setText(fileName);
}

//If the select button is pressed, the user has to select a directory/folder on the device
//where the file has to be transmitted to
void MainWindow::selectButtonPressed(void)
{
    //Start child dialog passing the socket reference
    TargetDirBrowser *browser;

    //first of all, read and show all errors from the queue
    tcpSocket->readErrorQueue(true);

    if (isLINUX)
        browser = new TargetDirBrowser(tcpSocket, "/var", this);
    else
        browser = new TargetDirBrowser(tcpSocket, "D:", this);

    if (browser->exec())
    {
        targetDir = browser->getTargetDir();
        ui->destinationDir->setText(targetDir);
    }

    delete browser;
}

//If the transmit button is pressed, the real work starts...
void MainWindow::transmitButtonPressed(void)
{
   //Everything ready?
    if (!isConnected)
    {
        QMessageBox::information(this, "SocketFileTransmitter",
                                 "Not connected!");
        return;
    }

    if (fileName.isEmpty())
    {
        QMessageBox::information(this, "SocketFileTransmitter",
                                 "No file selected!");
        return;
    }

    if (targetDir.isEmpty())
    {
        QMessageBox::information(this, "SocketFileTransmitter",
                                 "No target directory selected!");
        return;
    }

    //Ok, everything is ready now
    //store the filename w/o extension for later MMEM:DATA command
    QString targetFileName = fileName.section("/", -1);

//    tcpSocket->write("MMEM:CDIR 'E:\\'\n");
//    tcpSocket->write("MMEM:DATA 'testfile.txt'");
//    tcpSocket->write(", #243THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG\n");

    //Set target directory on instrument
    QString command = "MMEM:CDIR '" + targetDir + "'\n";
    QByteArray murx = command.toLatin1();
    const char *temp = murx.constData();
    qDebug(temp);

    tcpSocket->write(command, true);

    //the whole file is transmitted with one single command
    //first parameter is the name of the file, a comma and a space separator

    command = "MMEM:DATA '" + targetFileName + "', ";
    murx = command.toLatin1();
    temp = murx.constData();
    qDebug(temp);

    tcpSocket->write(command, true);

    // now open the file and determine its size
    QFile transmitFile(fileName);
    transmitFile.open(QIODevice::ReadOnly);

    qint64 size = transmitFile.size();
    int sizeSize = log10((double)size) + 1;

    //send the size info
    QString s1 = QString::number(size);
    command.sprintf("#%d", sizeSize);
    command += s1;

    murx = command.toLatin1();
    temp = murx.constData();
    qDebug(temp);
    tcpSocket->write(command, true);

    //set up progress bar (if needed)
    bool useProgressBar = (size > BLOCK_SIZE);
    int blockCount = 0;

    if (useProgressBar)
        ui->progressBar->setRange(0,size/BLOCK_SIZE);

    //start the Timer
    QTime t;
    t.start();

    //send the file contents
    qint64 readSize;
    do
    {
        char buffer[BLOCK_SIZE+1];
        readSize = transmitFile.read(buffer, BLOCK_SIZE);
        if (readSize)
            tcpSocket->write(buffer, readSize, true);

        if (useProgressBar)
        {
            blockCount++;
            ui->progressBar->setValue(blockCount);
        }
    }
    while (readSize);

    //terminate transmission
    tcpSocket->write("\n", 1, true);

    //close file
    transmitFile.close();

    //Reset progress bar
    ui->progressBar->reset();

    //Report success to status info
    QString statusInfo;

    int elapsedTimeMilliseconds = t.elapsed();
    if (elapsedTimeMilliseconds)
    {
        double elapsedTimeSeconds = elapsedTimeMilliseconds/1000.0;
        statusInfo.sprintf("%ld bytes have been transmitted in %lf seconds (%.3lf MBytes/s)",
                           (unsigned long) size, elapsedTimeSeconds, (double)size/1024.0/1024.0/elapsedTimeSeconds);
    }
    else //2fast2measure :-)
    {
        statusInfo.sprintf("%ld bytes have been transmitted too fast to measure",
                           (unsigned long) size);
    }


    ui->status->setPlainText(statusInfo);
}
